@@ -48,6 +48,7 @@ Functions
 ---------
 
 """
+from functools import partial
 from typing import Callable, Any, Union, Generator, Tuple, List
 
 import numpy as np
@@ -332,9 +333,11 @@ def rolling(
     >>> np.sum(arr, axis=1)
     array([nan,  3.,  5.,  7.,  9.])
     """
-    if not isinstance(window, int):
+    if not any(isinstance(window, t) for t in [int, np.integer]):
         raise TypeError(f'Wrong window type ({type(window)}) int expected')
 
+    window = int(window)
+
     if array.size < window:
         raise ValueError('array.size should be bigger than window')
 
@@ -342,11 +345,14 @@ def rolling(
         if not skip_na:
             prepend_func = prepend_na
             if np.issubdtype(array.dtype, np.datetime64):
-                prepend_func = lambda arr, n: np.hstack((np.repeat(np.datetime64('NaT'), n), arr))
+
+                def prepend_func(arr, n):
+                    return np.hstack((np.repeat(np.datetime64('NaT'), n), arr))
 
             yield from (prepend_func(array[:i + 1], (window - 1) - i) for i in np.arange(window - 1))
 
-        yield from (array[i:i + window] for i in np.arange(array.size - (window - 1)))
+        starts = np.arange(array.size - (window - 1))
+        yield from (array[start:end] for start, end in zip(starts, starts + window))
 
     return np.array([row for row in rows_gen()]) if as_array else rows_gen()
 
@@ -385,9 +391,11 @@ def rolling_apply(func: Callable, window: int, *arrays: np.ndarray, n_jobs: int
     >>> rolling_apply(func, 2, arr, arr2, k=-1)
     array([  nan,  -5.5,  -8.5, -11.5, -14.5])
     """
-    if not isinstance(window, int):
+    if not any(isinstance(window, t) for t in [int, np.integer]):
         raise TypeError(f'Wrong window type ({type(window)}) int expected')
 
+    window = int(window)
+
     if max(len(x.shape) for x in arrays) != 1:
         raise ValueError('Wrong array shape. Supported only 1D arrays')
 
@@ -395,14 +403,23 @@ def rolling_apply(func: Callable, window: int, *arrays: np.ndarray, n_jobs: int
         raise ValueError('Arrays must be the same length')
 
     def _apply_func_to_arrays(idxs):
-        return func(*[array[idxs.astype(np.int)] for array in arrays], **kwargs)
+        return func(*[array[idxs[0]:idxs[-1] + 1] for array in arrays], **kwargs)
 
-    rolls = rolling(np.arange(len(arrays[0])), window, skip_na=True)
+    array = arrays[0]
+    rolls = rolling(
+        array if len(arrays) == n_jobs == 1 else np.arange(len(array)),
+        window=window,
+        skip_na=True
+    )
 
     if n_jobs == 1:
-        arr = [_apply_func_to_arrays(idxs) for idxs in rolls]
+        if len(arrays) == 1:
+            arr = list(map(partial(func, **kwargs), rolls))
+        else:
+            arr = list(map(_apply_func_to_arrays, rolls))
     else:
-        arr = Parallel(n_jobs=n_jobs)(delayed(_apply_func_to_arrays)(idxs) for idxs in rolls)
+        f = delayed(_apply_func_to_arrays)
+        arr = Parallel(n_jobs=n_jobs)(f(idxs[[0, -1]]) for idxs in rolls)
 
     return prepend_na(arr, n=window - 1)
 
@@ -440,9 +457,11 @@ def expanding(
     array([array([1, 2, 3]), array([1, 2, 3, 4]), array([1, 2, 3, 4, 5])],
           dtype=object)
     """
-    if not isinstance(min_periods, int):
+    if not any(isinstance(min_periods, t) for t in [int, np.integer]):
         raise TypeError(f'Wrong min_periods type ({type(min_periods)}) int expected')
 
+    min_periods = int(min_periods)
+
     if array.size < min_periods:
         raise ValueError('array.size should be bigger than min_periods')
 
@@ -490,9 +509,11 @@ def expanding_apply(func: Callable, min_periods: int, *arrays: np.ndarray, n_job
     >>> expanding_apply(func, 2, arr, arr2, k=-1)
     array([  nan,  -5.5,  -9.5, -14.5, -20.5])
     """
-    if not isinstance(min_periods, int):
+    if not any(isinstance(min_periods, t) for t in [int, np.integer]):
         raise TypeError(f'Wrong min_periods type ({type(min_periods)}) int expected')
 
+    min_periods = int(min_periods)
+
     if max(len(x.shape) for x in arrays) != 1:
         raise ValueError('Supported only 1-D arrays')
 
@@ -502,10 +523,20 @@ def expanding_apply(func: Callable, min_periods: int, *arrays: np.ndarray, n_job
     def _apply_func_to_arrays(idxs):
         return func(*[array[idxs.astype(np.int)] for array in arrays], **kwargs)
 
-    rolls = expanding(np.arange(len(arrays[0])), min_periods, skip_na=True)
+    array = arrays[0]
+    rolls = expanding(
+        array if len(arrays) == n_jobs == 1 else np.arange(len(array)),
+        min_periods=min_periods,
+        skip_na=True
+    )
+
     if n_jobs == 1:
-        arr = [_apply_func_to_arrays(idxs) for idxs in rolls]
+        if len(arrays) == 1:
+            arr = list(map(partial(func, **kwargs), rolls))
+        else:
+            arr = list(map(_apply_func_to_arrays, rolls))
     else:
-        arr = Parallel(n_jobs=n_jobs)(delayed(_apply_func_to_arrays)(idxs) for idxs in rolls)
+        f = delayed(_apply_func_to_arrays)
+        arr = Parallel(n_jobs=n_jobs)(map(f, rolls))
 
     return prepend_na(arr, n=min_periods - 1)
